==> hestia_earth_models-0.62.3-py3-none-any.whl: hestia_earth/models/utils/emission.py <==
from collections.abc import Iterable
from typing import Optional, Union

from hestia_earth.schema import EmissionMethodTier, SchemaType
from hestia_earth.utils.api import download_hestia
from hestia_earth.utils.model import linked_node
from hestia_earth.utils.lookup import get_table_value, download_lookup, column_name

from . import _term_id, _include_methodModel, flatten_args
from .blank_node import find_terms_value
from .constant import Units, get_atomic_conversion


EMISSION_METHOD_TIERS = [e.value for e in EmissionMethodTier]


def _new_emission(term, model=None):
    node = {'@type': SchemaType.EMISSION.value}
    node['term'] = linked_node(term if isinstance(term, dict) else download_hestia(_term_id(term)))
    return _include_methodModel(node, model)


def is_in_system_boundary(term_id: str):
    lookup = download_lookup('emission.csv')
    value = get_table_value(lookup, 'termid', term_id, column_name('inHestiaDefaultSystemBoundary'))
    # handle numpy boolean
    return not (not value)


def get_nh3_no3_nox_to_n(cycle: dict, nh3_term_id: str, no3_term_id: str, nox_term_id: str, allow_none: bool = False):
    default_value = 0 if allow_none else None

    nh3 = find_terms_value(cycle.get('emissions', []), nh3_term_id, default=default_value)
    nh3 = None if nh3 is None else nh3 / get_atomic_conversion(Units.KG_NH3, Units.TO_N)
    no3 = find_terms_value(cycle.get('emissions', []), no3_term_id, default=default_value)
    no3 = None if no3 is None else no3 / get_atomic_conversion(Units.KG_NO3, Units.TO_N)
    nox = find_terms_value(cycle.get('emissions', []), nox_term_id, default=default_value)
    nox = None if nox is None else nox / get_atomic_conversion(Units.KG_NOX, Units.TO_N)

    return (nh3, no3, nox)


_EMISSION_METHOD_TIER_RANKING = [
    EmissionMethodTier.MEASURED,
    EmissionMethodTier.TIER_3,
    EmissionMethodTier.TIER_2,
    EmissionMethodTier.TIER_1,
    EmissionMethodTier.BACKGROUND,
    EmissionMethodTier.NOT_RELEVANT
]
"""
A ranking of `EmissionMethodTier`s from strongest to weakest.
"""


_EmissionMethodTiers = Union[EmissionMethodTier, str, Iterable[Union[EmissionMethodTier, str]]]
"""
A type alias for a single emission method tier, as either an EmissionMethodTier enum or string, or multiple emission
method tiers, as either an iterable of EmissionMethodTier enums or strings.
"""


def min_emission_method_tier(*methods: _EmissionMethodTiers) -> EmissionMethodTier:
    """
    Get the minimum ranking emission method tier from the provided methods.

    n.b., `max` function is used as weaker methods have higher indices.

    Parameters
    ----------
    *methods : EmissionMethodTier | str | Iterable[EmissionMethodTier] | Iterable[str]
        Emission method tiers or iterables of emission method tiers.

    Returns
    -------
    EmissionMethodTier
        The emission method tier method with the minimum ranking.
    """
    methods_ = [to_emission_method_tier(arg) for arg in flatten_args(methods)]
    return max(
        methods_,
        key=lambda method: _EMISSION_METHOD_TIER_RANKING.index(method),
        default=_EMISSION_METHOD_TIER_RANKING[-1]
    )


def to_emission_method_tier(method: Union[EmissionMethodTier, str]) -> Optional[EmissionMethodTier]:
    """
    Convert the input str to an `EmissionMethodTier` if possible.

    Parameters
    ----------
    method : EmissionMethodTier | str
        The emission method tier as either a `str` or `EmissionMethodTier`.

    Returns
    -------
    EmissionMethodTier | None
        The matching `EmissionMethodTier` or `None` if invalid string.
    """
    return (
        method if isinstance(method, EmissionMethodTier)
        else EmissionMethodTier(method) if method in EMISSION_METHOD_TIERS
        else None
    )

==> hestia_earth_models-0.64.8-py3-none-any.whl: hestia_earth/models/utils/emission.py <==
from collections.abc import Iterable
from typing import Optional, Union
from hestia_earth.schema import EmissionMethodTier, SchemaType, TermTermType
from hestia_earth.utils.api import download_hestia
from hestia_earth.utils.model import linked_node
from hestia_earth.utils.lookup import get_table_value, download_lookup, column_name

from . import _term_id, _include_methodModel, flatten_args
from .blank_node import find_terms_value
from .constant import Units, get_atomic_conversion


EMISSION_METHOD_TIERS = [e.value for e in EmissionMethodTier]


def _new_emission(term, model=None):
    node = {'@type': SchemaType.EMISSION.value}
    node['term'] = linked_node(term if isinstance(term, dict) else download_hestia(_term_id(term)))
    return _include_methodModel(node, model)


def is_in_system_boundary(term_id: str):
    lookup = download_lookup('emission.csv')
    value = get_table_value(lookup, 'termid', term_id, column_name('inHestiaDefaultSystemBoundary'))
    # handle numpy boolean
    return not (not value)


def get_nh3_no3_nox_to_n(cycle: dict, nh3_term_id: str, no3_term_id: str, nox_term_id: str, allow_none: bool = False):
    default_value = 0 if allow_none else None

    nh3 = find_terms_value(cycle.get('emissions', []), nh3_term_id, default=default_value)
    nh3 = None if nh3 is None else nh3 / get_atomic_conversion(Units.KG_NH3, Units.TO_N)
    no3 = find_terms_value(cycle.get('emissions', []), no3_term_id, default=default_value)
    no3 = None if no3 is None else no3 / get_atomic_conversion(Units.KG_NO3, Units.TO_N)
    nox = find_terms_value(cycle.get('emissions', []), nox_term_id, default=default_value)
    nox = None if nox is None else nox / get_atomic_conversion(Units.KG_NOX, Units.TO_N)

    return (nh3, no3, nox)


_EMISSION_METHOD_TIER_RANKING = [
    EmissionMethodTier.MEASURED,
    EmissionMethodTier.TIER_3,
    EmissionMethodTier.TIER_2,
    EmissionMethodTier.TIER_1,
    EmissionMethodTier.BACKGROUND,
    EmissionMethodTier.NOT_RELEVANT
]
"""
A ranking of `EmissionMethodTier`s from strongest to weakest.
"""


_EmissionMethodTiers = Union[EmissionMethodTier, str, Iterable[Union[EmissionMethodTier, str]]]
"""
A type alias for a single emission method tier, as either an EmissionMethodTier enum or string, or multiple emission
method tiers, as either an iterable of EmissionMethodTier enums or strings.
"""


def min_emission_method_tier(*methods: _EmissionMethodTiers) -> EmissionMethodTier:
    """
    Get the minimum ranking emission method tier from the provided methods.

    n.b., `max` function is used as weaker methods have higher indices.

    Parameters
    ----------
    *methods : EmissionMethodTier | str | Iterable[EmissionMethodTier] | Iterable[str]
        Emission method tiers or iterables of emission method tiers.

    Returns
    -------
    EmissionMethodTier
        The emission method tier method with the minimum ranking.
    """
    methods_ = [to_emission_method_tier(arg) for arg in flatten_args(methods)]
    return max(
        methods_,
        key=lambda method: _EMISSION_METHOD_TIER_RANKING.index(method),
        default=_EMISSION_METHOD_TIER_RANKING[-1]
    )


def to_emission_method_tier(method: Union[EmissionMethodTier, str]) -> Optional[EmissionMethodTier]:
    """
    Convert the input str to an `EmissionMethodTier` if possible.

    Parameters
    ----------
    method : EmissionMethodTier | str
        The emission method tier as either a `str` or `EmissionMethodTier`.

    Returns
    -------
    EmissionMethodTier | None
        The matching `EmissionMethodTier` or `None` if invalid string.
    """
    return (
        method if isinstance(method, EmissionMethodTier)
        else EmissionMethodTier(method) if method in EMISSION_METHOD_TIERS
        else None
    )


def filter_emission_inputs(emission: dict, term_type: TermTermType):
    inputs = emission.get('inputs', [])
    return [i for i in inputs if i.get('termType') == term_type.value]
